/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006-2010 Smithsonian Astrophysical Observatory
 *
 * This file is part of aperture
 *
 * aperture is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

#include "config.h"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>


#include "luav.h"

#include <suplib/str.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "params.h"

#include "luasup.h"
#include "lua_register.h"

#include "aperture.h"

#include "common.h"

#include "lua_iface.h"

/*---------------------------------------------------------------------*/

static char *
stack_top_msg( lua_State *L )
{
  const char *topmsg;
  char *msg;

  tf_assert( lua_gettop( L ) && (topmsg = lua_tostring(L, -1)) );
  tf_assert( msg = str_dup( topmsg ) );
  return msg;
}

static
int panic( lua_State *L ) {

    tf_die( stack_top_msg( L ) );
    return 1;
}


/*---------------------------------------------------------------------*/

void exec_lua( Params *io )
{
    lua_State *L;
    tf_enter( "exec_lua" );

    /* create interpreter instance */
    if ( NULL == ( L = luaL_newstate() ) )
	tf_die( "error creating Lua interpreter" );

    /* register a new panic function */
    lua_atpanic( L, panic );

    /* register standard lua libraries */
    luaL_openlibs( L );

    /* register aperture functions */
    luaopen_aperture( L );

    if ( io->cfgfile ) {

	lua_getglobal(L, "saotrace" );
	lua_getfield( L, -1, "aperture" );
	lua_remove( L, -2 );
	lua_getfield( L, -1, "init" );
	lua_remove( L, -2 );

        lua_pushstring( L, io->cfgfile );
        lua_pushstring( L, io->cfgpars );
        lua_call( L, 2, 0 );
    }

    lua_close( L );


  tf_leave();
}
