// --8<--8<--8<--8<--
//
// Copyright (C) 2011 Smithsonian Astrophysical Observatory
//
// This file is part of bpmanip
//
// bpmanip is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <vector>
#include <string>

using std::string;
using std::vector;

#include "gtest/gtest.h"

#include "Fields.h"

class FieldsTest : public ::testing::Test {

 protected:
    virtual void SetUp() {

	str_fields.push_back( string( "a" ) );
	str_fields.push_back( string( "a1" ) );
	str_fields.push_back( string( "a2" ) );
	str_fields.push_back( string( "b" ) );
	str_fields.push_back( string( "b1" ) );
	str_fields.push_back( string( "b2" ) );
	str_fields.push_back( string( "c" ) );
	str_fields.push_back( string( "d" ) );

    }

    vector<string> str_fields;
    vector<string> include;
    vector<string> exclude;

};

TEST_F( FieldsTest, CheckNull ) {

    Fields fields( include, exclude );

    vector<string> result = fields.allowed( str_fields );

    EXPECT_EQ( str_fields, result );

}

TEST_F( FieldsTest, CheckInclude1 ) {

    include.push_back( string("a?" ) );

    Fields fields( include, exclude );

    vector<string> result = fields.allowed( str_fields );

    ASSERT_EQ( 2, result.size() );

    EXPECT_EQ( "a1", result[0] );
    EXPECT_EQ( "a2", result[1] );
}

TEST_F( FieldsTest, CheckInclude2 ) {

    include.push_back( string( "b*" ) );

    Fields fields( include, exclude );

    vector<string> result = fields.allowed( str_fields );

    ASSERT_EQ( 3, result.size() );


    EXPECT_EQ( "b",  result[0] );
    EXPECT_EQ( "b1", result[1] );
    EXPECT_EQ( "b2", result[2] );

}

TEST_F( FieldsTest, CheckIncludeExclude ) {

    include.push_back( string( "b*" ) );
    exclude.push_back( string( "b" ) );

    Fields fields( include, exclude );

    vector<string> result = fields.allowed( str_fields );

    ASSERT_EQ( 2, result.size() );

    EXPECT_EQ( "b1", result[0] );
    EXPECT_EQ( "b2", result[1] );

}
