// --8<--8<--8<--8<--
//
// Copyright (C) 2011 Smithsonian Astrophysical Observatory
//
// This file is part of bpmanip
//
// bpmanip is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// -->8-->8-->8-->8--

#include <vector>
#include <string>
#include <cstdarg>

using std::string;
using std::vector;

#include "gtest/gtest.h"

#include "Fields.h"

void
str_fill( vector<string>& strs, const char *strings[] ) {

    for ( ; *strings ; strings++ ) {

	strs.push_back( string( *strings ) );
    }
}

vector<string>&
str_fill( vector<string>& strs, ... ) {

    va_list ap;
    va_start( ap, strs );

    for ( ; ; ) {

	const char* p = va_arg( ap, const char * );
	if ( p == 0 ) break;

	strs.push_back( string( p ) );
    }
    va_end( ap );

    return strs;
}

class FieldsTest : public ::testing::Test {

 protected:
    virtual void SetUp() {

	str_fill( str_fields,
		  "a",
		  "a1",
		  "a2",
		  "b",
		  "b1",
		  "b2",
		  "c",
		  "d",
		  NULL );
    }

    vector<string> str_fields;
    vector<string> results;
    vector<string> expected;
    vector<string> include;
    vector<string> exclude;

};

TEST_F( FieldsTest, CheckNull ) {

    Fields fields( include, exclude );

    vector<string> result = fields.rejected( str_fields );

    EXPECT_EQ( 0, result.size() );

}

TEST_F( FieldsTest, CheckInclude1 ) {

    include.push_back( string("a?" ) );

    Fields fields( include, exclude );

    results = fields.rejected( str_fields );

    str_fill( expected, "a", "b", "b1", "b2", "c", "d", NULL );

    EXPECT_EQ( expected, results );
}

TEST_F( FieldsTest, CheckInclude2 ) {

    include.push_back( string( "b*" ) );

    Fields fields( include, exclude );

    results = fields.rejected( str_fields );
    str_fill( expected, "a", "a1", "a2", "c", "d", NULL );
    EXPECT_EQ( expected, results );
}

TEST_F( FieldsTest, CheckIncludeExclude ) {

    include.push_back( string( "b*" ) );
    exclude.push_back( string( "b" ) );

    Fields fields( include, exclude );

    results = fields.rejected( str_fields );
    str_fill( expected, "a", "a1", "a2", "b", "c", "d", NULL );
    EXPECT_EQ( expected, results );
}
