require 'lunatest'

units = require 'saotrace.raygen.units'

local math = require( 'math' )
local pi = math.pi
local deg2rad = pi / 180
local rad2deg = 180 / pi


function test_normalize_value()

   local system = units.system

   -- value + string unit, no default unit

   local v, u = units.normalize_value( { 1, 'degree' } )

   assert_equal( 1, v )
   assert_true( u:are_convertible( units.angular.degree ) )
   assert_equal( 1, u:get_converter(units.angular.degree):convert( v ) )

   -- value + string unit, default unit


   local v, u = units.normalize_value( { 1, 'degree' }, units.linear.meter )

   assert_equal( 1, v )
   assert_true( u:are_convertible( units.angular.degree ) )
   assert_equal( 1, u:get_converter(units.angular.degree):convert( v ) )

   -- value + ut_unit unit, no default unit

   local v, u = units.normalize_value( { 1, units.angular.degree } )

   assert_equal( 1, v )
   assert_true( u:are_convertible( units.angular.degree ) )
   assert_equal( 1, u:get_converter(units.angular.degree):convert( v ) )

   -- value + ut_unit unit, no default unit

   local v, u = units.normalize_value( { 1, units.angular.degree }, units.linear.meter )

   assert_equal( 1, v )
   assert_true( u:are_convertible( units.angular.degree ) )
   assert_equal( 1, u:get_converter(units.angular.degree):convert( v ) )


   -- value, no default unit

   local v, u = units.normalize_value( 1 )

   assert_equal( 1, v )
   assert_equal( nil, u )

   -- value, default unit

   local v, u = units.normalize_value( 1, units.angular.degree )

   assert_equal( 1, v )
   assert_true( u:are_convertible( units.angular.degree ) )
   assert_equal( 1, u:get_converter(units.angular.degree):convert( v ) )

end
lunatest.run()
