// -*-C++-*-
#ifndef JITTER_TABLE_FILE_H
#define JITTER_TABLE_FILE_H

#include <vector>
#include <string>

#include "rTime.h"
#include "Jitter/Common.h"

namespace Jitter {

    namespace TableFile {

	class Base {

	public:
	    size_t read( Point*& points, raygen::TimeInterval& tvalid );

	    // number of rows to read at once.  use only during testing
	    long  rowsize;

	    // name of column containing time;
	    string time_col_name;

	protected:
	    // name of file
	    string file;

	    Base( string& file ) :
		rowsize(0),
		time_col_name("time"),
		file( file )
	    { }

	    virtual size_t read_fits( Point*& points, raygen::TimeInterval& tvalid ) = 0;
	};

	class Quaternion : public Base {

	public:
	    Quaternion( string& file ) :
		Base( file ),
		quat_col_name( "q_att" ) {}

	    string quat_col_name;

	    size_t read_fits( Point*& points, raygen::TimeInterval& tvalid );

	};

	class Euler : public Base {

	public:

	    // rotation axes in order of application when reading in
	    // Euler angles
	    std::vector<EulerAxis*> axes;

	    Euler( string& file ) :
		Base( file )
	    {}

	    ~Euler( ) {

		std::vector< EulerAxis* >::iterator iaxis;

		for( iaxis = axes.begin(); iaxis < axes.end() ; iaxis++ )
		    delete *iaxis;
	    }

	    size_t read_fits( Point*& points, raygen::TimeInterval& tvalid );
	};

    }

}

#endif // ! JITTER_TABLE_FILE_H
