#ifndef RAYGEN_CONFIG_H
#define RAYGEN_CONFIG_H

#include <lua.hpp>
#include <lauxlib.h>

#include <tracefctxx/TraceFct.h>

#include "lua_stack.h"

class SysConf {

 public:

    SysConf( lua_State *L, const string& obj_name ) : _L(L), _obj_name(obj_name) {

	if ( get_luaglobal( _L, _obj_name ) )
	    TraceFct::die( "error accessing %s\n", _obj_name.c_str() );

	reference = luaL_ref(_L, LUA_REGISTRYINDEX );
    }

    ~SysConf( ){

	if ( _L )
	    luaL_unref(_L, LUA_REGISTRYINDEX, reference );

    }

    bool get( const string& field, double* value = NULL ) {

	bool is_valid = get_value( field );
	if ( is_valid && value )
	    *value = lua_tonumber(_L, -1 );
	lua_pop( _L, -1 );
	return is_valid;
    }

    double get_double( const string& field ) {
	if ( ! get_value( field ) )
	    TraceFct::die( "error accessing %s.%s\n", _obj_name.c_str(), field.c_str() );
	double val = lua_tonumber( _L, -1 );
	lua_pop( _L, -1 );
	return val;
    }

    bool get_bool( const string& field ) {
	if ( ! get_value( field ) )
	    TraceFct::die( "error accessing %s.%s\n", _obj_name.c_str(), field.c_str() );
	bool val = lua_toboolean( _L, -1 );
	lua_pop( _L, -1 );
	return val;
    }

    bool get_value( const string& field ) {
	lua_rawgeti( _L, LUA_REGISTRYINDEX, reference );
	if ( get_luafield( _L, field.c_str() ) ) {
	    lua_pop( _L, -1 );
	    lua_pushnil( _L );
	}
	else {
	    lua_replace( _L, -2 );
	}
	return ! lua_isnil( _L, -1 );
    }

 private:

    lua_State *_L;
    string _obj_name;
    int  reference;
};

#endif // ! RAYGEN_CONFIG_H
