module( ..., package.seeall )

local table = require( 'table' )
local os = require( 'os' )

local srcdir = os.getenv('srcdir') or '.'

local db = table.concat( { srcdir, 'data', 'geometry.rdb'}, '/' )


local giter = require('saotrace.geometry').read{ file = db, mode = 'iterator' }

local optic = {}

for geo in giter do

   if not optic[geo.optic] then
      optic[geo.optic] = {}
   end

   optic[geo.optic][geo.shell] = geo

end

local ea = require( 'saotrace.raygen.ea.ideal' ).entrance_aperture


function setup ()
   _G.setup()
   saotrace.raygen.intrinsic.stash = {}
end

function test_outermost( )

   ea{ shell = 1, file = db }

   local stash = saotrace.raygen.intrinsic.stash
   assert_equal( optic.p[1].rho_f, stash[1][1].opts.ro )
   assert_equal( optic.p[3].rho_f, stash[1][1].opts.ri )

end

function test_middle( )

   ea{ shell = 3, file = db }

   local stash = saotrace.raygen.intrinsic.stash
   assert_equal( optic.p[3].rho_f, stash[1][1].opts.ro )
   assert_equal( optic.p[4].rho_f, stash[1][1].opts.ri )

end

function test_inner( )

   ea{ shell = 6, file = db }

   local stash = saotrace.raygen.intrinsic.stash
   assert_equal( optic.p[6].rho_f, stash[1][1].opts.ro )
   assert_equal( optic.p[6].rho_f - (optic.p[4].rho_f - optic.p[6].rho_f),
		 stash[1][1].opts.ri, 'p4-p6' )
end

