module( ..., package.seeall )

local table = require( 'table' )
local os = require( 'os' )

local srcdir = os.getenv('srcdir') or '.'

local picket_db = table.concat( { srcdir, 'data', 'picket.rdb'}, '/' )

local raygen = require('saotrace.raygen')
local intrinsic = require('saotrace.raygen.intrinsic')
local utils = require( 'saotrace.raygen.spectrum.utils' )
local vspec = require( 'saotrace.raygen.spectrum.vspec'):new()

local va = require('validate.args')
local validate = va.validate

local spectra
local spec = { vspec:vspec() }


function setup()

   _G.setup()
   spectra = utils.Spectra:new( )
   va.opts{ udata = { spectra = spectra } }
   intrinsic.stash = {}

end

function test_object ( )

   local nspec = vspec:new()

   nspec:add( 'foobar', {} )
   nspec:delete( 'foobar' )


end



function test_spec_discrete_1 ()

   local ok, val = validate( spec, 2.3 )

   assert_true( ok, val )
   val = spectra:spectra()

   assert_equal( 2.3, val[1].args.energy , "posnum" )

end

function test_spec_discrete_n ()

   local ok, val = validate( spec, { 1.2, 2.3, 3.4 } )

   assert_true( ok, val )

   val = spectra:spectra()
   assert_equal( 1.2, val[1].args.energy )
   assert_equal( 1,   val[1].args.flux )

   assert_equal( 2.3, val[2].args.energy )
   assert_equal( 1,   val[2].args.flux )

   assert_equal( 3.4, val[3].args.energy )
   assert_equal( 1,   val[3].args.flux )

end

function test_spec_fluxed ()

   local ok, val = validate( spec, { { 1.2, 2},
				     { 2.3, 3},
				     { 3.4, 4},
				  }
			  )
   assert_true( ok, val )

   val = spectra:spectra()
   assert_equal( 1.2, val[1].args.energy )
   assert_equal( 2,   val[1].args.flux )

   assert_equal( 2.3, val[2].args.energy )
   assert_equal( 3,   val[2].args.flux )

   assert_equal( 3.4, val[3].args.energy )
   assert_equal( 4,   val[3].args.flux )

end

function test_spec_picket ()

   local ok, val = validate( spec, {
				{ file = picket_db,
				  type = 'picket'
			       }
			     }
			  )
   assert_true( ok, val )

   assert_equal( picket_db, val[1].file )
   assert_equal( 'picket', val[1].type )

   val = spectra:spectra()
   assert_equal( 1, val[1].args.energy, "posnum" )
   assert_equal( 1, val[1].args.flux, "posnum" )
   assert_equal( 2, val[2].args.energy, "posnum" )
   assert_equal( 2, val[2].args.flux, "posnum" )
   assert_equal( 3, val[3].args.energy, "posnum" )
   assert_equal( 3, val[3].args.flux, "posnum" )

end

function test_spec_flat ()

   local ok, val = validate( spec, {
				{ emin = 1,
				  emax = 2,
				  flux = 3,
			       }
			     }
			  )
   assert_true( ok, val )
   val = spectra:spectra()

   assert_equal( 1, val[1].args.emin )
   assert_equal( 2, val[1].args.emax )
   assert_equal( 3, val[1].args.flux )


end

function test_spec_file ()

   local ok, val = validate( spec, {
				{ file = 'foo.rdb',
				  format = 'rdb',
				  units = 'photons/s/cm2',
			       }
			     }
			  )
   assert_true( ok, val )
   val = spectra:spectra()

   assert_equal( 'foo.rdb', val[1].args.file )


end
